//---------------------------------------------------------------------------
// Falling pieces by dllee (Dong-Liang Lee) http://dllee.ktop.com.tw
// <2004-09-11> First release
// <2004-09-12> Bugfixed, pieces falling and gathering perfectly [:D]
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "FallingUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
Graphics::TBitmap *pBitmap;
Byte *FallingPtn[8];
Byte *Ptn[10];
Byte *PtnIndex;
bool  Started;
bool  ShowInSteps;
bool  EXIT;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
  : TForm(Owner)
{
  int ReOrder[8]={8,1,5,3,4,7,2,6};
                //0,1,2,3,4,5,6,7
  int ReOrderIndex[8]={1,6,3,4,2,7,5,0};
  pBitmap = new Graphics::TBitmap();
  pBitmap->PixelFormat=pf24bit;
  pBitmap->Width=256;
  pBitmap->Height=128;
  for(int i=0;i<10;i++)
  {
    Ptn[i]=new Byte[256*3];
    memset(Ptn[i],0x10,256*3);
    for(int j=0;j<256;j++)
      if(j%3==0)
        memset(&Ptn[i][j*3],0,3);
  }
  for(int i=0;i<256;i++)
  {
    if(i%3==0) continue;
    Ptn[ReOrder[(i/3)%8]][i*3]=0xFF;
    // *(*(Ptn+*(ReOrder+(i/3)%8))+i*3)=0xFF;
    Ptn[9][i*3]=0xFF;
  }
  for(int i=0;i<8;i++)
  {
    FallingPtn[i]=new Byte[256*3];
    memcpy(FallingPtn[i],Ptn[0],256*3);
    for(int j=0;j<=i;j++)
    {
      for(int k=0;k<256;k++)
      {
        if(k%3==0) continue;
        if((k/3)%8==ReOrderIndex[j])
          FallingPtn[i][k*3]=0xFF;
      }
    }
  }
  PtnIndex=new Byte[128];
  memset(PtnIndex,0,128);
  this->Color=clBlack;
  Started=false;
}
//---------------------------------------------------------------------------
__fastcall TForm1::~TForm1()
{
  delete [] PtnIndex;
  for(int i=0;i<8;i++)  delete [] FallingPtn[i];
  for(int i=0;i<10;i++) delete [] Ptn[i];
  delete pBitmap;
}
//---------------------------------------------------------------------------
// Steps  Finished  Remark
// 0-62       -1    Initial falling
// 63         0     FirstBar touch down
// 63~70     0~1    FirstEnds touch down
// 70         1     SecondBar touch down
// 70~77     1~2    SecondEnds touch down
// 77         2     3rdBar touch down
// 77~84     2~3    3rdEnds touch down
// 63+F*7     F     FrdBar touch down
// 63+F*7     F
// ~63+F*7+7  ~F+1  FrdEnds touch down
void __fastcall TForm1::Button1Click(TObject *Sender)
{
  if(Started) return;
  Started=true;

  ShowInSteps=false;
  EXIT=false;
  memset(PtnIndex,0,128);
  int iFinished;
  for(int i=0;i<64+64*7 && !EXIT;i++)
  {
    if(i>=63) iFinished=((i-63)/7);   // iFinished=0~64
    else      iFinished=-1;
    if(iFinished>0)                   // Moving Bars by moving PtnIndex[]
    {
      if((iFinished+1)*2<128)
        memmove(&PtnIndex[iFinished*2],&PtnIndex[(iFinished+1)*2],128-(iFinished+1)*2);
    }
    else
      memmove(&PtnIndex[0],&PtnIndex[2],128-2);
    PtnIndex[127]=(i)%8+1;            // Add New Falling Bar
    for(int i=0;i<iFinished;i++)      // Show Finished Bar
    {
      PtnIndex[i*2]=0;
      PtnIndex[i*2+1]=9;
    }
    for(int j=1;j<=128;j++)           // Show Falling Bars
    {
      memcpy((Byte *)pBitmap->ScanLine[128-j],
        Ptn[PtnIndex[j-1]],256*3);
    }
    if(iFinished>=0 && (128-iFinished*2-2)>=0)  // Show Finished Bars
      memcpy((Byte *)pBitmap->ScanLine[128-iFinished*2-2],
          FallingPtn[(i)%7],256*3);

    Canvas->Draw(15,15,pBitmap);      // Show Bitmap Image
    Application->ProcessMessages();   // Check if other buttons are pressed
    if(ShowInSteps)                   // Show in steps
    {                                 // press one [step] button for one step
      ShowInSteps=false;
      while(!ShowInSteps && !EXIT)
      {
        Application->ProcessMessages();
        Sleep(100);
      }
    }
    Sleep(5);
  }
  Started=false;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Button2Click(TObject *Sender)
{
  EXIT=true;
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Button3Click(TObject *Sender)
{
  ShowInSteps=!ShowInSteps;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Button4Click(TObject *Sender)
{
  for(int i=0;i<10;i++)               // Show Ptn[]
    for(int j=7*i;j<7*(i+1);j++)
      memcpy((Byte *)pBitmap->ScanLine[j],
        Ptn[i],256*3);
  for(int i=0;i<8;i++)                // Show FallingPtn[]
    for(int j=7*(i+10);j<7*(i+11);j++)
      memcpy((Byte *)pBitmap->ScanLine[j],
        FallingPtn[i],256*3);

  for(int i=0;i<18;i++)               // Show Separate(Empty) Bar
    memcpy((Byte *)pBitmap->ScanLine[i*7],
        Ptn[0],256*3);

  Canvas->Draw(15,15,pBitmap);        // Show Bitmap image
  Canvas->Font->Color=clWhite;        // Show hints
  Canvas->Font->Name="Terminal";      // "Small Fonts";
  Canvas->Font->Size=6;
  for(int i=0;i<10;i++)
    Canvas->TextOut(235,15+7*i,String("Ptn[")+i+String("]"));
  for(int i=10;i<18;i++)
    Canvas->TextOut(200,15+7*i,String("FallingPtn[")+(i-10)+String("]"));
}
//---------------------------------------------------------------------------

